# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"say_hello": "00_core.ipynb",
         "load_kssl": "01_data.loading.ipynb",
         "PRIMARY_COLOR": "03_vis.core.ipynb",
         "ACCENT_COLOR": "03_vis.core.ipynb",
         "DEFAULT_STYLE": "03_vis.core.ipynb",
         "centimeter": "03_vis.core.ipynb",
         "set_style": "03_vis.core.ipynb",
         "plot_spectra": "03_vis.core.ipynb",
         "summary_plot": "03_vis.core.ipynb",
         "plot_validation_curve": "03_vis.core.ipynb",
         "plot_learning_curve": "03_vis.core.ipynb",
         "plot_capacity": "03_vis.core.ipynb",
         "select_y": "04_data.selection.ipynb",
         "select_tax_order": "04_data.selection.ipynb",
         "select_X": "04_data.selection.ipynb",
         "get_y_by_order": "04_data.selection.ipynb",
         "log_transform_y": "05_data.transform.ipynb",
         "CO2_REGION": "05_data.transform.ipynb",
         "TakeDerivative": "05_data.transform.ipynb",
         "SNV": "05_data.transform.ipynb",
         "DropSpectralRegions": "05_data.transform.ipynb",
         "rpd": "07_training.metrics.ipynb",
         "rpiq": "07_training.metrics.ipynb",
         "stb": "07_training.metrics.ipynb",
         "mape": "07_training.metrics.ipynb",
         "lccc": "07_training.metrics.ipynb",
         "eval_reg": "07_training.metrics.ipynb",
         "compute_valid_curve": "10_training.plsr.ipynb",
         "PLS_model": "10_training.plsr.ipynb",
         "Evaluator": "12_training.cnn.ipynb",
         "Model": "12_training.cnn.ipynb",
         "weights_init": "12_training.cnn.ipynb",
         "Learner": "12_training.cnn.ipynb",
         "SpectralDataset": "13_data.torch.ipynb",
         "DataLoaders": "13_data.torch.ipynb",
         "SNV_transform": "13_data.torch.ipynb",
         "Noop": "13_data.torch.ipynb"}

modules = ["core.py",
           "data/loading.py",
           "vis/core.py",
           "data/selection.py",
           "data/transform.py",
           "training/metrics.py",
           "training/plsr.py",
           "training/cnn.py",
           "data/torch.py"]

doc_url = "https://franckalbinet.github.io/mirzai/"

git_url = "https://github.com/franckalbinet/mirzai/tree/master/"

def custom_doc_links(name): return None
