# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/01_data.core.ipynb (unless otherwise specified).

__all__ = ['load_file']

# Cell
import numpy as np
import os
from fastcore.test import *

# Cell
def load_file():
    return np.array([1, 2, 3])

# Cell
# def load_file(name, path='./data', loader=np.load):
#     """
#     Loads individual data file.

#     Args:
#         name: the name of the file
#         path: the path containing
#         data
#         loader: the function used to load. E.g. `np.load` or `pickle.load`

#     Returns:
#         data as numpy.ndarray or Dict (when pickle is used)
#     """

#     with open(os.path.join(path, name), 'rb') as f:
#         return loader(f)